package com.poststats.rs.api;

import java.util.List;

import com.poststats.rs.api.model.Facility;
import com.poststats.rs.api.model.PagedCollection;
import com.poststats.rs.api.model.Pagination;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import jakarta.validation.constraints.DecimalMax;
import jakarta.validation.constraints.DecimalMin;
import jakarta.validation.constraints.Max;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import jakarta.validation.constraints.Size;
import jakarta.ws.rs.BeanParam;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;

/**
 * @author brian.long@poststats.com
 */
@Path("/facilities")
@Tag(name = "Facility API")
@Produces({ MediaType.APPLICATION_JSON, Constants.V1_JSON })
@ApiResponse(responseCode = "200", description = "Success", content = {
        @Content(mediaType = MediaType.APPLICATION_JSON),
        @Content(mediaType = Constants.V1_JSON)
})
@ApiResponse(responseCode = "400", description = "A request parameter is not valid")
@ApiResponse(responseCode = "404", description = "No matching facilities were found")
public interface FacilitiesApi {

    @GET
    @Path("/byName")
    @Operation(summary = "Searches for facilities by their name.", description = "Searches for facilities by their name, excluding prefix (e.g. The) and suffix (e.g. Golf Club), returning limited meta-data about each matched facility.")
    @Tag(name = "Search API")
    PagedCollection<List<Facility>> searchByName(
            @Parameter(description = "A fragment or whole facility name; the prefix and suffix are not included", example = "Bushwood") @QueryParam("name") @NotNull @Size(min = 3) String name,
            @BeanParam @Valid Pagination paging);

    @GET
    @Path("/byLocation/{country}")
    @Operation(summary = "Searches for facilities by their location.", description = "Searches for facilities by their country, returning limited meta-data about each matched facility.")
    @Tag(name = "Search API")
    PagedCollection<List<Facility>> searchByJurisdiction(
            @Parameter(description = "A country code", example = "US") @PathParam("country") @NotNull String country,
            @BeanParam @Valid Pagination paging);

    @GET
    @Path("/byLocation/{country}/{state}")
    @Operation(summary = "Searches for facilities by their location.", description = "Searches for facilities by their major jurisdiction (e.g. US States), returning limited meta-data about each matched facility.")
    @Tag(name = "Search API")
    PagedCollection<List<Facility>> searchByJurisdiction(
            @Parameter(description = "A country code", example = "US") @PathParam("country") @NotNull String country,
            @Parameter(description = "A State or high-level jurisdiction", example = "FL") @PathParam("state") @NotNull String state,
            @BeanParam @Valid Pagination paging);

    @GET
    @Path("/byGeo")
    @Operation(summary = "Searches for facilities by their location.", description = "Searches for facilities by their major jurisdiction (e.g. US States), returning limited meta-data about each matched facility.")
    @Tag(name = "Search API")
    PagedCollection<List<Facility>> searchByJurisdiction(
            @Parameter(description = "A latitude in decimal degrees", example = "41.2390") @QueryParam("latitude") @NotNull @DecimalMin("-90.0") @DecimalMax("90.0") double latitude,
            @Parameter(description = "A longitude in decimal degrees", example = "-81.34891") @QueryParam("longitude") @NotNull @DecimalMin("-180.0") @DecimalMax("180.0") double longitude,
            @Parameter(name = "radius", description = "A search radius in miles", example = "15") @QueryParam("radius") @Positive @Max(100) Integer radiusInMiles,
            @BeanParam @Valid Pagination paging);

}
